{% extends 'layout.html' %}

{% block title %}{{ escape(page.title) }}{% end %}
{% block location %}{{ page.location }}-section{% end %}

{% block content %}
  {% if page.toc %}
  <section class="toc">
    {{ page.toc }}
  </section>
  {% end %}

  <article class="page">
    <header>
      <h1>{{ escape(page.title) }}</h1>
    </header>

    {{ page.content }}
  </article>
{% end %}
