{% extends 'layout.html' %}

{% block title %}404{% end %}

{% block content %}
<p>It appears that the page you're looking for does not exist. Or not anymore.</p>

<p><a href="{{ get_url('/') }}">Index</a>.</p>
{% end %}
