@import "elements.less";

// @font-sans: 'Droid Sans', 'DejaVu Sans', 'Helvetica Neue', 'Helvetica', Arial, sans-serif;
// @font-serif: 'Droid Serif', Georgia, 'Bitstream Charter', 'Times New Roman', serif;
// @font-mono: 'Droid Sans Mono', monospace;


@font-sans: "Liberation Sans", "Nimbus Sans L", "FreeSans", "Helvetica Neue", Helvetica, Arial, sans-serif;
@font-serif: "Gentium Book Basic", "Cambo", "Bitstream Charter", "URW Palladio L", Palatino, "Palatino Linotype", "Book Antiqua", serif;
@font-mono: 'Envy Code R', 'Droid Sans Mono', 'Bitstream Vera Sans Mono', 'DejaVu Sans Mono', Inconsolata, Consolas, monospace;

html, body, div, span, object, iframe,
h1, h2, h3, h4, h5, h6, p, blockquote, pre,
abbr, address, cite, code,
del, dfn, em, img, ins, kbd, q, samp,
small, strong, sub, sup, var,
b, i,
dl, dt, dd, ol, ul, li,
fieldset, form, label, legend,
table, caption, tbody, tfoot, thead, tr, th, td,
article, aside, canvas, details, figcaption, figure,
footer, header, hgroup, menu, nav, section, summary,
time, mark, audio, video {
  margin: 0;
  padding: 0;
  border: 0;
  outline: 0;
  font-size: 100%;
  vertical-align: baseline;
  background: transparent;
}

::-moz-selection { background: #000;color: #fff;text-shadow: none }
::selection { background: #000;color: #fff;text-shadow: none }

pre ::-webkit-scrollbar {
  width: 12px;
}

pre ::-webkit-scrollbar-track {
  -webkit-box-shadow: inset 0 0 6px rgba(0,0,0,0.3);
  .rounded(10px);
}

pre ::-webkit-scrollbar-thumb {
  -webkit-box-shadow: inset 0 0 6px rgba(0,0,0,0.5);
  .rounded(10px);
}

html {
  -webkit-font-smoothing: subpixel-antialiased;
}

body {
  // [style], [variant], [weight], size[/line-height], family
  font: 1em/1.5em @font-sans;
  padding: 50px;
}

article, aside, details, figcaption, figure,
footer, header, hgroup, menu, nav, section {
  display: block;
}

nav ul {
  list-style: none;
}

blockquote, q {
  quotes: none;
}

blockquote:before, blockquote:after,
q:before, q:after {
  content: '';
  content: none;
}

sup {
  font-size: xx-small;
  vertical-align: super;
}

a {
  margin: 0;
  padding: 0;
  font-size: 100%;
  vertical-align: baseline;
  background: transparent;

  &:link, &:visited { color: #556270 }
  &:hover, &:active { color: #C44D58 }
}

/* change colours to suit your needs */
ins {
  background-color: #ff9;
  color: #000;
  text-decoration: none;
}

/* change colours to suit your needs */
mark {
  background-color: #ff9;
  color: #000;
  font-style: italic;
  font-weight: bold;
}

del {
  text-decoration:  line-through;
}

abbr[title], dfn[title] {
  border-bottom: 1px dotted;
  cursor: help;
}

table {
  border-collapse: collapse;
  border-spacing: 0;
}

th, td {
  padding: .5em 1em;
}
th {
  text-align: left;
  font-weight: bold;
  border-bottom: 2px solid #999;
}
td {
  border-bottom: 1px solid #ddd;
}

/* change border colour to suit your needs */
hr {
  display: block;
  height: 1px;
  border: 0;
  border-top: 1px solid #cccccc;
  margin: 1em 0;
  padding: 0;
}

input, select {
  vertical-align: middle;
}

h1, h2, h3, h4, h5, h6 {
  font-family: @font-serif;
}

p {
  margin: 1.5em 0;

  img {
    max-width: 100%;
    outline: 2px solid #ddd;
  }
}

ol, ul {
  margin: 1.5em 0;
  padding-left: 2em;

  ol, ul {
    margin-top: 0;
    margin-bottom: 0;
    padding-left: 1.3em;
  }

  p { margin-top: 0 }
}

ul {
  list-style: none;
  padding-left: .5em;

  ul { padding-left: .3em }
  li { padding-left: 1em }

  li:before {
    content: "\00BB \0020";
    margin-left: -.9em;
    float: left;
  }
}

code {
  background-color: #fee9cc;
  color: rgba(0,0,0,0.75);
  padding: 1px 3px;
  font-size: .9em;

  .rounded(3px);
}
pre {
  margin: 1.5em 0 1.5em;
  padding: .5em .7em;
  font-size: .9em;

  .rounded(4px);
}
pre, code {
  font-family: @font-mono;
}
pre code {
  display: block;
  padding: 0 0 .3em;
  overflow-x: auto;
  background: transparent;
  color: inherit;
  font-size: 1em;
}

#navigation {
  h1 {
    font-size: 2em;
    margin: 0 0 1em;
  }

  a {
    // display: block;
    padding: .5em;
    color: #444;
    text-decoration: none;

    // .transition(0.1s, linear);

    &:hover, &:active {
      background: #eee;
      color: #333;
    }
  }
}

#index-section #nav-index,
#changelog-section #nav-changelog,
#manual-section #nav-manual,
#api-section #nav-api {
  background: #eee;
  color: #333;
}

#content {
  margin: 4em 0 0;

  h1 {
    margin: 0 0 .3em;
    font-size: 1.8em;

    a:link, a:visited {
      color: #000;
      text-decoration: none;
    }

    a:hover, a:active {
      color: #C44D58;
      text-decoration: underline;
    }
  }

  h2 { font-size: 1.5em }
  h3 { font-size: 1.2em }
  h2, h3, h4, h5, h6 {
    margin-top: 2em;
  }

  &:after{
    clear: both;
    content: ".";
    display: block;
    height: 0;
    visibility: hidden;
    font-size: 0;
  }
}

.toc {
  float: left;
  font-size: .9em;
  padding-right: 50px;

  ul:first-child {
    margin-top: .1em;
    margin-bottom: 5em;
  }

  a {
    text-decoration: none;
  }
}

.page {
  max-width: 600px;
  float: left;
}

#bottom {
  clear: both;
  color: #555;
  font-size: .8em;

  a { color: #555 }
}


/* --- Pygments: https://github.com/uraimo/pygments-vimstyles/blob/master/wombat.css --- */

.highlight  { background: #242424; color: #f6f3e8}
.highlight .hll { background-color: #ffffcc }
.highlight .c { color: #99968b; font-style: italic} /* Comment */
.highlight .err { color: #f6f3e8} /* Error */
.highlight .g { color: #f6f3e8} /* Generic */
.highlight .k { color: #8ac6f2} /* Keyword */
.highlight .l { color: #f6f3e8} /* Literal */
.highlight .n { color: #f6f3e8} /* Name */
.highlight .o { color: #f6f3e8} /* Operator */
.highlight .x { color: #f6f3e8} /* Other */
.highlight .p { color: #f6f3e8} /* Punctuation */
.highlight .cm { color: #99968b; font-style: italic} /* Comment.Multiline */
.highlight .cp { color: #e5786d} /* Comment.Preproc */
.highlight .c1 { color: #99968b; font-style: italic} /* Comment.Single */
.highlight .cs { color: #99968b; font-style: italic} /* Comment.Special */
.highlight .gd { color: #f6f3e8} /* Generic.Deleted */
.highlight .ge { color: #f6f3e8} /* Generic.Emph */
.highlight .gr { color: #f6f3e8} /* Generic.Error */
.highlight .gh { color: #f6f3e8; font-weight: bold} /* Generic.Heading */
.highlight .gi { color: #f6f3e8} /* Generic.Inserted */
.highlight .go { color: #808080; background-color: #303030 } /* Generic.Output */
.highlight .gp { color: #f6f3e8} /* Generic.Prompt */
.highlight .gs { color: #f6f3e8} /* Generic.Strong */
.highlight .gu { color: #f6f3e8; font-weight: bold} /* Generic.Subheading */
.highlight .gt { color: #f6f3e8} /* Generic.Traceback */
.highlight .kc { color: #8ac6f2} /* Keyword.Constant */
.highlight .kd { color: #8ac6f2} /* Keyword.Declaration */
.highlight .kn { color: #8ac6f2} /* Keyword.Namespace */
.highlight .kp { color: #8ac6f2} /* Keyword.Pseudo */
.highlight .kr { color: #8ac6f2} /* Keyword.Reserved */
.highlight .kt { color: #cae682} /* Keyword.Type */
.highlight .ld { color: #f6f3e8} /* Literal.Date */
.highlight .m { color: #e5786d} /* Literal.Number */
.highlight .s { color: #95e454; font-style: italic} /* Literal.String */
.highlight .na { color: #cae682} /* Name.Attribute */
.highlight .nb { color: #f6f3e8} /* Name.Builtin */
.highlight .nc { color: #f6f3e8} /* Name.Class */
.highlight .no { color: #e5786d} /* Name.Constant */
.highlight .nd { color: #f6f3e8} /* Name.Decorator */
.highlight .ni { color: #e7f6da} /* Name.Entity */
.highlight .ne { color: #f6f3e8} /* Name.Exception */
.highlight .nf { color: #cae682} /* Name.Function */
.highlight .nl { color: #f6f3e8} /* Name.Label */
.highlight .nn { color: #f6f3e8} /* Name.Namespace */
.highlight .nx { color: #f6f3e8} /* Name.Other */
.highlight .py { color: #f6f3e8} /* Name.Property */
.highlight .nt { color: #8ac6f2} /* Name.Tag */
.highlight .nv { color: #cae682} /* Name.Variable */
.highlight .ow { color: #f6f3e8} /* Operator.Word */
.highlight .w { color: #f6f3e8} /* Text.Whitespace */
.highlight .mf { color: #e5786d} /* Literal.Number.Float */
.highlight .mh { color: #e5786d} /* Literal.Number.Hex */
.highlight .mi { color: #e5786d} /* Literal.Number.Integer */
.highlight .mo { color: #e5786d} /* Literal.Number.Oct */
.highlight .sb { color: #95e454; font-style: italic} /* Literal.String.Backtick */
.highlight .sc { color: #95e454; font-style: italic} /* Literal.String.Char */
.highlight .sd { color: #95e454; font-style: italic} /* Literal.String.Doc */
.highlight .s2 { color: #95e454; font-style: italic} /* Literal.String.Double */
.highlight .se { color: #95e454; font-style: italic} /* Literal.String.Escape */
.highlight .sh { color: #95e454; font-style: italic} /* Literal.String.Heredoc */
.highlight .si { color: #95e454; font-style: italic} /* Literal.String.Interpol */
.highlight .sx { color: #95e454; font-style: italic} /* Literal.String.Other */
.highlight .sr { color: #95e454; font-style: italic} /* Literal.String.Regex */
.highlight .s1 { color: #95e454; font-style: italic} /* Literal.String.Single */
.highlight .ss { color: #95e454; font-style: italic} /* Literal.String.Symbol */
.highlight .bp { color: #f6f3e8} /* Name.Builtin.Pseudo */
.highlight .vc { color: #cae682} /* Name.Variable.Class */
.highlight .vg { color: #cae682} /* Name.Variable.Global */
.highlight .vi { color: #cae682} /* Name.Variable.Instance */
.highlight .il { color: #e5786d} /* Literal.Number.Integer.Long */
